/* Autonomous Satellite Tracker.
 *
 * Connections on Adafruit ESP8266 Huzzah:
 *   Servo controller connects to SDA and SCL
 *   GPS TX connects to 16, RX to 2
 *
 */


#include "AutoSatTracker-ESP.h"
#include "NV.h"
#include "Sensor.h"
#include "Circum.h"
#include "Gimbal.h"
#include "Target.h"
#include "Webpage.h"
#include "Wire.h"
#include "SSD1306Wire.h"

NV *nv;
Sensor *sensor;
Circum *circum;
Gimbal *gimbal;
Target *target;
Webpage *webpage;

//Globale Variablen die in anderen Modulen gesetzt werden
extern bool gps_lock;
extern bool track_status;
extern String satname;

const int oled_contrast = 64;

//Display settings for 0.96 OLED display
const int I2C_DISPLAY_ADDRESS = 0x3c;
#if defined(ESP8266)
//const int SDA_PIN = D3;
//const int SDC_PIN = D4;
const int SDA_PIN = D2; // same as I2C pins for the rest
const int SDC_PIN = D1; 
#else
const int SDA_PIN = 5; //D3;
const int SDC_PIN = 4; //D4;
#endif

SSD1306Wire     display(I2C_DISPLAY_ADDRESS, SDA_PIN, SDC_PIN);

/* called first and once
 */
void
setup()
{

    // init serial monitor
    Serial.begin (115200);
    delay(500);

    // this just resets the soft timeout, the hard timeout is still 6 seconds
    ESP.wdtDisable();

    // scan for I2C devices
    resetWatchdog();
    Serial.println();
    Serial.print (F("Scanning I2C:"));
    Wire.begin(); // original code did not have Wire.begin and I2C scan was not working
    for (uint8_t a = 1; a < 127; a++) {
	    Wire.beginTransmission(a);
	    if (Wire.endTransmission(true) == 0) 
      {
	      Serial.print (F(" ")); Serial.print (a,HEX);
      }
    }
    Serial.println();

    // instantiate each module
    
    // initialize display
    Serial.println(F("Making OLED"));
    display.init();
    display.clear();
    display.display();

    //  display.flipScreenVertically();
    display.setFont(ArialMT_Plain_24);
    display.setTextAlignment(TEXT_ALIGN_CENTER);
    display.setContrast(oled_contrast);
    display.drawString(64,0,"Welcome");
    display.drawString(64,19,"to");
    display.drawString(64,39,"SatTrack");
    display.display();
    //show welcome screen until main loop starts
    // change font for main loop display
    display.setFont(ArialMT_Plain_10);
    
    Serial.println (F("making NV"));
    nv = new NV();
    Serial.println (F("making Sensor"));
    sensor = new Sensor();
    Serial.println (F("making Circum"));
    circum = new Circum();
    Serial.println (F("making Gimbal"));
    gimbal = new Gimbal();
    Serial.println (F("making Target"));
    target = new Target();
    Serial.println (F("making Webpage"));
    webpage = new Webpage();
    
    // all set to go.. see you in loop()
    Serial.println (F("Ready"));

}

//Update the OLED Status display
void OLED_update()
{

  String statestring = "";
  char st[64];
  
  resetWatchdog();
  display.clear();

  //Display Title
  display.drawString(64, 0, "SatTrack");

  // Display Sat Name if tracking is active and Tracking State (on or off)
  
  if (target->track_status)
    statestring = target->satname + " tracking";
  else
    statestring = target->satname + " not tracking";
  display.drawString(64, 12, statestring);

  //Display GPS state
  if (circum->nsats !=0xff)
    sprintf(st,"%d",circum->nsats);
  else
    sprintf(st,"%s","INVAL"); 
  if (circum->gps_lock)
    statestring = "GPS locked " + (String)st + " satellites";
  else
    statestring = "GPS UNLOCKED";
  display.drawString(64, 24, statestring);
  
  //Display Gimbal state
  if (gimbal->connected())
  {
    if (gimbal->calibrated())
       statestring="Gimbal ok calibrated";
    else
       statestring="Gimbal ok UNCALIBRATED";
  }
  else
    statestring="GIMBAL MISSING";
  display.drawString(64,36, statestring);
  
  float Gimbal_AZ = 0;
  float Gimbal_EL = 0;
  //Display Azimuth and Elevation of the antenne as derived from sensor
  sensor->getAzEl (&Gimbal_AZ, &Gimbal_EL);
  sprintf(st,"AZ: %3.1f°     EL: %3.1f°",Gimbal_AZ,Gimbal_EL);
  display.drawString(64,48, (String)st);

  //Write everyhting to the display
  display.display();
}

/* called repeatedly forever
 */
void
loop()
{
    // check for ethernet activity
    webpage->checkEthernet();

    // check for new GPS info
    circum->checkGPS();

    // follow the target
    target->track();

    //update Oled Display
    OLED_update();
}

void
resetWatchdog()
{
    ESP.wdtFeed();
    yield();
}
